/* pt1-pci.c: A PT1 on PCI bus driver for Linux. */
#define DRV_NAME	"pt1-pci"
#define DRV_VERSION	"1.00"
#define DRV_RELDATE	"11/28/2008"

static int debug = 1;			/* 1 normal messages, 0 quiet .. 7 verbose. */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/interrupt.h>

#include <asm/system.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include	"pt1_i2c_data.h"
#include	"pt1_chdata.h"
#include	"pt1_init_data.h"

/* These identify the driver base version and may not be removed. */
static char version[] __devinitdata =
KERN_INFO DRV_NAME ".c:v" DRV_VERSION " " DRV_RELDATE " \n";

MODULE_AUTHOR(" ");
MODULE_DESCRIPTION("PCI earthsoft PT1 driver");
MODULE_LICENSE("GPL");

module_param(debug, int, 0);
MODULE_PARM_DESC(debug, "debug level (1-2)");

static struct pci_device_id pt1_pci_tbl[] = {
	{ 0x10ee, 0x211a, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ 0, }
};
MODULE_DEVICE_TABLE(pci, pt1_pci_tbl);


typedef	struct	_pt1_device{
	unsigned long	mmio_start ;
	__u32			mmio_len ;
	void __iomem		*regs;
}PT1_DEVICE;

PT1_DEVICE	dev_conf ;
/*
    ǡ
*/
static void pt1_scan(PT1_DEVICE *pt1)
{
	int		lp ;
	int		lp2 ;
	int		lp3 ;
	PT1_INIT_DATA	*ptr ;
	__u32			val ;
	__u32			val2;

	for(lp = 0 ; lp < PT1_SCAN_DATA_ALL_NUM ; lp++){
		ptr = pt1_scan_table[lp].ptr ;
		while(ptr->trans_ptr != NULL){
			for(lp2 = 0 ; lp2 < ptr->trans_num ; lp2++){
				writel(ptr->trans_ptr[lp2], (dev_conf.regs + 0x010));
			}
			writel(0x04, dev_conf.regs);
			val2 = 0xffffffff ;
			for(lp3 = 0 ; lp3 < 100 ; lp3++){
				val = readl(dev_conf.regs);
				if(val == val2){
					break ;
				}
				val2 = val ;
				printk(KERN_NOTICE "(%d)(1)%08x\n", (lp + 1), val);
			}
			ptr = ptr + 1 ;
		}
		val2 = 0xffffffff ;
		for(lp3 = 0 ; lp3 < 100 ; lp3++){
			val = readl((dev_conf.regs + 0x08));
			if(val == val2){
				break ;
			}
			val2 = val ;
			printk(KERN_NOTICE "(%d)(2)%08x\n", (lp + 1), val);
		}
	}
}

static int __devinit pt1_pci_init_one (struct pci_dev *pdev,
				     const struct pci_device_id *ent)
{
	__u32			val ;
	int				rc ;
	int				lp ;
	int				lp2 ;
	int				lp3 ;
	int				b_f ;
	int				val2 ;

	rc = pci_enable_device (pdev);
	if (rc)
		return rc;

	dev_conf.mmio_start = pci_resource_start(pdev, 0);
	dev_conf.mmio_len = pci_resource_len(pdev, 0);
	dev_conf.regs = ioremap(dev_conf.mmio_start, dev_conf.mmio_len);
	if (!dev_conf.regs) {
		printk(KERN_ERR "pt1: Can't remap register area.\n");
		goto out_err_regbase;
	}
	val = 0 ;
	//  
	while(1){
		if(val == 0XFFC0000){
			break ;
		}
		writel(val, dev_conf.regs + 0x10);
		val += 0x40000;
	}

	for(lp = 0 ; lp < 127; lp++){
		writel(0, dev_conf.regs + 0x10);
	}
	b_f = 1 ;
	for(lp = 0 ; lp < 127 ; lp++){
		val = readl(dev_conf.regs);
		val2 = readl(dev_conf.regs);
		if(val != val2){
			continue ;
/*
		}else if(val == 0x00000020){
 */
		}else{
			b_f = 0 ;
			break ;
		}
	}
	if(b_f){
		printk(KERN_ERR "Init Error 1(%x)\n", val);
		goto out_err_fpga;
	}

	for(lp = 0 ; lp < 2 ; lp++){
		val = readl(dev_conf.regs);
		printk(KERN_NOTICE "addr 0: %08x\n", val);
	}
	for(lp = 0 ; lp < 114 ; lp++){
		writel(8, dev_conf.regs);
		b_f = 1 ;
		for(lp2 = 0 ; lp2 < 128 ; lp2++){
			val = readl(dev_conf.regs);
			val2 = readl(dev_conf.regs);
			if(val != val2){
				continue ;
/*
			}else if(val == 0x80000020){
 */
			}else{
				b_f = 0 ;
				break ;
			}
		}
		if(b_f){
			printk(KERN_ERR "Init Error 2(%x)\n", val);
			goto out_err_fpga;
		}
	}

	writel(0x1010000, dev_conf.regs);
	writel(0x1000000, dev_conf.regs);

	b_f = 1 ;
	for(lp = 0 ; lp < 184 ; lp++){
		val = readl(dev_conf.regs);
		val2 = readl(dev_conf.regs);
		if(val != val2){
			continue ;
/*
		}else if(val == 0x80000021){
 */
		}else{
			b_f = 0 ;
			break ;
		}
		printk(KERN_NOTICE "addr 0: %08x(writel after4)\n", val);
	}
	if(b_f){
		printk(KERN_ERR "Init Error 3(%x)\n", val);
		goto out_err_fpga;
	}

	writel(0x2020000, dev_conf.regs);
	writel(0x2000000, dev_conf.regs);

	b_f = 1 ;
	for(lp = 0 ; lp < 184 ; lp++){
		val = readl(dev_conf.regs);
		val = readl(dev_conf.regs);
		if(val != val2){
			continue ;
/*
		}else if(val == 0x80000023){
 */
		}else{
			b_f = 0 ;
			break ;
		}
	}
	if(b_f){
		printk(KERN_ERR "Init Error 4(%x)\n", val);
		goto out_err_fpga;
	}

	for(lp = 0 ; lp < 128 ; lp++){
		writel(0x02, dev_conf.regs);
		for(lp2 = 0 ; lp2 < 10000 ; lp2++){
			val = readl(dev_conf.regs);
			// ¿ʬդ
			if(val == 0x80000023){
				break ;
			}
		}
		b_f = 1 ;
		for(lp2 = 0 ; lp2 < 10000 ; lp2++){
			val = readl(dev_conf.regs);
			if(val == 0x80000027){
				b_f = 0 ;
				break ;
			}
		}
/*
		if(b_f){
			printk(KERN_ERR "Init Error 6(%x:%d)\n", val, lp);
			goto out_err_fpga;
		}
 */
	}

	// ǡž
	// ǡ֥å
	for(lp = 0 ; lp < PT1_INIT_1_NUM ; lp++){
		writel(pt1_init_1[lp], (dev_conf.regs + 0x08));
	}
	// ǡ֥å
	for(lp = 0 ; lp < PT1_INIT_2_NUM ; lp++){
		writel(pt1_init_2[lp], (dev_conf.regs + 0x04));
	}

	// ֥å֥åޤ
	for(lp = 0 ; lp <PT1_INIT_DATA_NUM ; lp++){
		// ǡ֥å
		for(lp2 = 0 ; lp2 < pt1_init_all[lp].trans_num ; lp2++){
			writel(pt1_init_all[lp].trans_ptr[lp2], (dev_conf.regs + 0x010));
		}

		writel(0x04, dev_conf.regs);
		for(lp3 = 0 ; lp3 < 100 ; lp3++){
			val = readl(dev_conf.regs);
			if(val == 0x800000a7){
				break ;
			}
		}
		for(lp3 = 0 ; lp3 < 100 ; lp3++){
			val = readl((dev_conf.regs + 0x08));
			if(val == 0x0000410a){
				break ;
			}
			printk(KERN_NOTICE "(%d)(%d)addr 8: %08x(Data Transfer4 after)\n", (lp + 1), lp3, val);
		}
	}

	// ǽλ
	writel(0x101, (dev_conf.regs + 0x08));
	writel(0x202, (dev_conf.regs + 0x08));
	writel(0x404, (dev_conf.regs + 0x08));
	writel(0x0808, (dev_conf.regs + 0x08));

	writel(0x0c, (dev_conf.regs + 0x0c));
	writel(0xd0, (dev_conf.regs + 0x0c));
	writel(0xe00, (dev_conf.regs + 0x0c));
	writel(0xf000, (dev_conf.regs + 0x0c));


	writel(0xb0b0000, dev_conf.regs);
	writel(0, dev_conf.regs + 4);

	pt1_scan(&dev_conf);
	iounmap(dev_conf.regs);
	return 0;
out_err_fpga:
	writel(0xb0b0000, dev_conf.regs);
	writel(0, dev_conf.regs + 4);
	iounmap(dev_conf.regs);
	return -ENODEV;
out_err_regbase:
	return -ENODEV;

}

static void __devexit pt1_pci_remove_one(struct pci_dev *pdev)
{

}


static int pt1_pci_open(struct net_device *dev)
{
	return 0;
}

static int pt1_pci_close(struct net_device *dev)
{
	return 0;
}


#ifdef CONFIG_PM
static int pt1_pci_suspend (struct pci_dev *pdev, pm_message_t state)
{
	return 0;
}

static int pt1_pci_resume (struct pci_dev *pdev)
{
	return 0;
}

#endif /* CONFIG_PM */


static struct pci_driver pt1_driver = {
	.name		= DRV_NAME,
	.probe		= pt1_pci_init_one,
	.remove		= __devexit_p(pt1_pci_remove_one),
	.id_table	= pt1_pci_tbl,
#ifdef CONFIG_PM
	.suspend	= pt1_pci_suspend,
	.resume		= pt1_pci_resume,
#endif /* CONFIG_PM */

};


static int __init pt1_pci_init(void)
{
	return pci_register_driver(&pt1_driver);
}


static void __exit pt1_pci_cleanup(void)
{
	pci_unregister_driver (&pt1_driver);
}

module_init(pt1_pci_init);
module_exit(pt1_pci_cleanup);
